/* SQL Database Interface. Main module
   Author: adv (Andriy Marchenko), 06 oct, 2001
           <adv@adv.org.ua>  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "types.h"
#include "sdbi.h"

#ifdef USE_PGSQL
#  include "postgres.h"
#endif

#ifdef USE_MYSQL
#  include "mysql.h"
#endif

#ifdef USE_ORACLE
#  include "oracle.h"
#endif


API_SDBI sql_t*
sql_init(const char* driver)
{
  sql_t* drv = (sql_t*)malloc(sizeof(sql_t));
  if (!drv)
    {
      perror("malloc");
      return NULL;
    }

  drv->opt      = NULL;
  drv->res_last = NULL;

  if (false);
#ifdef USE_MYSQL
  else if (strcasecmp(driver, "mysql") == 0)
    {
      drv->connect     = sql_mysql_connect;
      drv->disconnect  = sql_mysql_disconnect;
      drv->freeres     = sql_mysql_freeres;
      drv->execute     = sql_mysql_execute;
      drv->fetch       = sql_mysql_fetch;
      drv->getfield    = sql_mysql_getfield;
      drv->getfield_pos= sql_mysql_getfield_pos;
      drv->fieldlen    = sql_mysql_fieldlen;
      drv->lastrec     = sql_mysql_lastrec;

      drv->numrows    = sql_mysql_numrows;
      drv->numcols    = sql_mysql_numcols;
      drv->fieldname  = sql_mysql_fieldname;
    }
#endif /* USE_MYSQL  */
#ifdef USE_ORACLE
  else if (strcasecmp(driver, "oracle") == 0)
    {
      drv->connect     = sql_oracle_connect;
      drv->disconnect  = sql_oracle_disconnect;
      drv->freeres     = sql_oracle_freeres;
      drv->execute     = sql_oracle_execute;
      drv->fetch       = sql_oracle_fetch;
      drv->getfield    = sql_oracle_getfield;
      drv->getfield_pos= sql_oracle_getfield_pos;
      drv->fieldlen    = sql_oracle_fieldlen;
      drv->lastrec     = sql_oracle_lastrec;
      drv->numrows     = sql_oracle_numrows;
      drv->numcols     = sql_oracle_numcols;
      drv->fieldname   = sql_oracle_fieldname;
    }
#endif /* USE_ORACLE  */
#ifdef USE_PGSQL
  else if (strcasecmp(driver, "postgres") == 0)
    {
      drv->connect     = sql_postgres_connect;
      drv->disconnect  = sql_postgres_disconnect;
      drv->freeres     = sql_postgres_freeres;
      drv->execute     = sql_postgres_execute;
      drv->fetch       = sql_postgres_fetch;
      drv->getfield    = sql_postgres_getfield;
      drv->getfield_pos= sql_postgres_getfield_pos;
      drv->fieldlen    = sql_postgres_fieldlen;
      drv->lastrec     = sql_postgres_lastrec;
      drv->numrows     = sql_postgres_numrows;
      drv->numcols     = sql_postgres_numcols;
      drv->fieldname   = sql_postgres_fieldname;
    }
#endif /* USE_PGSQL  */
  else
    {
//TODO msg driver no found
      free(drv);
      drv = NULL;
    }

  return drv;
}


API_SDBI void
sql_close(sql_t* drv)
{
  sql_opt_t* tmp_opt = drv->opt;

  while (tmp_opt)
    {
      drv->opt = tmp_opt->next;
      free(tmp_opt);
      tmp_opt = drv->opt;
    }

  free(drv);
}


API_SDBI __uint16_t
sql_set_opt(sql_t* drv, const char* keyword, const char* value)
{
  sql_opt_t* new_opt = (sql_opt_t*)malloc(sizeof(sql_opt_t));

  new_opt->keyword = keyword;
  new_opt->value   = value;
  if (value)
      new_opt->ivalue = atol(value);
  else
      new_opt->ivalue = 0;
  new_opt->next    = drv->opt;

  drv->opt = new_opt;

  return 1;
}


API_SDBI const char*
sql_get_opt(sql_t* drv, const char* keyword)
{
  sql_opt_t* opt = drv->opt;

  while (opt != NULL)
    {
      if (strcmp(opt->keyword, keyword) == 0)
          return opt->value;

      opt = opt->next;
    }

  return NULL;
}


API_SDBI __int32_t
sql_get_iopt(sql_t* drv, const char* keyword)
{
  sql_opt_t* opt = drv->opt;

  while (opt != NULL)
    {
      if (strcmp(opt->keyword, keyword) == 0)
          return opt->ivalue;

      opt = opt->next;
    }

  return 0;
}


API_SDBI __uint16_t
sql_connect(sql_t* drv)
{
  return drv->connect(drv);
}


API_SDBI void
sql_disconnect(sql_t* drv)
{
  sql_res_t* res = drv->res_last;

  while (res)
      res = sql_freeres(res);

  drv->disconnect(drv);
}


API_SDBI sql_res_t*
sql_execute(sql_t* drv, const char* query)
{
  sql_res_t* res;

  if (!drv->connection)
      return NULL;

  if (!(res = (sql_res_t*)malloc(sizeof(sql_res_t))))
      return NULL;

res->q = query;
//printf("NEW: RES (%s)\n", res->q);

  res->drv = drv;
  res = drv->execute(res, query);

  if (!res->result)
    {
//printf("EMPTY RES (INSERT?):\n");
      free(res);
      return NULL;
    }

//printf("IS THIS SELECT ?\n");

  res->prev   = drv->res_last;
  res->next   = NULL;
  if (res->prev)
      res->prev->next = res;

  res->drv->res_last = res;

  return res;
}


API_SDBI sql_res_t*
sql_freeres(sql_res_t* res)
{
  sql_res_t* ret = 0;

  if (!res)
      return 0;

  res->drv->freeres(res);

  if (res->prev)
      res->prev->next = res->next;
  if (res->next)
      res->next->prev = res->prev;
  else
      res->drv->res_last = res->prev;

  if (res->prev)
      ret = res->prev;
  else
      ret = res->next;

//printf("DEL: RES (%s)\n", res->q);
  free(res);

  return ret;
}


API_SDBI __int16_t
sql_fetch(sql_res_t* res)
{
  if (!res)                  return 0;
  if (!res->result)          return 0;
  if (!res->drv)             return 0;
  if (!res->drv->connection) return 0;

  return res->drv->fetch(res);
}


API_SDBI const char*
sql_getfield_pos(sql_res_t* res, __uint32_t field)
{
  return res->drv->getfield_pos(res, field);
}


API_SDBI const char*
sql_getfield(sql_res_t* res, const char* field)
{
  return res->drv->getfield(res, field);
}


API_SDBI __uint32_t
sql_fieldlen(sql_res_t* res, __uint32_t field)
{
  return res->drv->fieldlen(res, field);
}


API_SDBI __uint32_t
sql_lastrec(sql_t* drv)
{
  return drv->lastrec(drv);
}


API_SDBI __uint32_t
sql_numrows(sql_res_t* res)
{
  return res->drv->numrows(res);
}


API_SDBI __uint32_t
sql_numcols(sql_res_t* res)
{
  return res->drv->numcols(res);
}


API_SDBI const char*
sql_fieldname(sql_res_t* res, __uint32_t field)
{
  return res->drv->fieldname(res, field);
}
